//*****************************************************************************
//
//	LCD-Modul  mit HD74780 Controller 
//
//	PORTD - Daten 4Bit PD4..7, LCD D0..3 auf GND
//	PD2 Enable	
//	PD3 RS
//	    WR auf GND 
//
//*****************************************************************************
#include <avr/io.h>
#include <stdint.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include "lcd.h"
#include "timer0.h"



//*****************************************************************************
//	LCD - Init
//  Input:  -	
//	Output:	-
//*****************************************************************************
void lcd_init(void)
{
   DDRD |= 0xFC;	
   PORTD &= 0x03;
   
   timer0_delay(150);
   lcd_rs(0);
   lcd_enable(0);
   asm("nop");
   lcd_enable(1);
   lcd_Data_H(0x20);
   asm("nop");
   asm("nop");
   lcd_enable(0);  
   timer0_delay(20); 

   lcd_write(INSTRUCTION,FUNCTION_SET);
#ifdef OLDLCD	   
   timer0_delay(15);
#endif
#ifndef OLDLCD
   timer0_delay(10);      
#endif
   lcd_write(INSTRUCTION,DISPLAY_ON);
#ifdef OLDLCD	   
   timer0_delay(15);
#endif
#ifndef OLDLCD
   timer0_delay(10);      
#endif
   lcd_write(INSTRUCTION,DISPLAY_CLEAR);
#ifdef OLDLCD	   
   timer0_delay(150);
#endif
#ifndef OLDLCD
   timer0_delay(100);      
#endif


   lcd_write(INSTRUCTION,SECOND_LINE);
   timer0_delay(10);
}


//*****************************************************************************
//	LCD - Enable
//  Input:  0/1
//	Output:	-
//*****************************************************************************
void lcd_enable(unsigned char uc_x)
{
   if (uc_x) PORTD |= 0x04; 
        else PORTD &= 0xFB;
}


//*****************************************************************************
//	LCD - RS
//  Input:  0/1
//	Output:	-
//*****************************************************************************
void lcd_rs(unsigned char uc_x)
{
   if (uc_x) PORTD |= 0x08; 
        else PORTD &= 0xF7;
}

//*****************************************************************************
//	LCD - Daten H
//  Input:  HByte
//	Output:	-
//*****************************************************************************
void lcd_Data_H(unsigned char uc_x)
{
   PORTD &= 0x0F;
   PORTD |= (uc_x & 0xF0);	
}


//*****************************************************************************
//	LCD - Daten L
//  Input:  HByte
//	Output:	-
//*****************************************************************************
void lcd_Data_L(unsigned char uc_x)
{
   lcd_Data_H(uc_x << 4);	
}


//*****************************************************************************
//	LCD - Byte schreiben
//  Input:  Typ(Befehl/Daten), Byte	
//	Output:	-
//*****************************************************************************
void lcd_write(unsigned char uc_datentyp,unsigned char uc_x)
{
   lcd_enable(0);
   if (uc_datentyp == INSTRUCTION) lcd_rs(0);
	                          else lcd_rs(1);

   lcd_enable(0);
   lcd_Data_H(uc_x);		        
   asm("nop");
   lcd_enable(1);
   asm("nop");
   asm("nop");
   lcd_enable(0);

   asm("nop");
   asm("nop");

   lcd_Data_L(uc_x);
   asm("nop");
   lcd_enable(1);
   asm("nop");
   asm("nop");
   lcd_enable(0);
   
   if (uc_datentyp == INSTRUCTION) timer0_delay(10);
     else timer0_delay(1);
}


//*****************************************************************************
//	LCD - Zeichen erzeugen
//  Input:  Zeichenadresse 0..7,Pattern 8 Byte	
//	Output:	-
//*****************************************************************************
void lcd_createchar(unsigned char uc_adr,char * pc_pattern)
{
unsigned char uc_cgram  = 0;
unsigned char uc_cnt;

   if (uc_adr < 8)
    {
       uc_cgram  = uc_adr * 8;
    } 
   
    lcd_write(INSTRUCTION,ENTRY_OFF | 0x02);
    lcd_write(INSTRUCTION,CGRAM | uc_cgram);
	
	for (uc_cnt=0; uc_cnt < 8; uc_cnt++)
	 {
        lcd_write(DATA,pc_pattern[uc_cnt]);
	 }
	
    lcd_write(INSTRUCTION,ENTRY_OFF); 
}


//*****************************************************************************
//	LCD - Zeichen schreiben
//  Input:  Zeichen,Zeile,Startposition	
//	Output:	-
//*****************************************************************************
void lcd_writechar(unsigned char uc_x,unsigned char uc_zeile,
                   unsigned char uc_offset)
{
  if (uc_offset > LINECHARS-1) uc_offset = 0;
  if ((uc_zeile > LINES) || (uc_zeile == 0)) uc_zeile = 1;
  
  if (uc_zeile == 1)
   {
     lcd_write(INSTRUCTION,FIRST_LINE + uc_offset);
   }

  if (uc_zeile == 2)
   {
     lcd_write(INSTRUCTION,SECOND_LINE + uc_offset);
   }	

  if (uc_zeile == 3)
   {
     lcd_write(INSTRUCTION,THIRD_LINE + uc_offset);
   }

  if (uc_zeile == 4)
   {
     lcd_write(INSTRUCTION,FOURTH_LINE + uc_offset);
   }
   
#ifdef OLDLCD	   
       timer0_delay(5);
#endif
   
  if (uc_zeile < 5)
   {  
      lcd_write(DATA,uc_x);
   } 		
}

				   
//*****************************************************************************
//	LCD - String schreiben
//  Input:  String,Zeile,Startposition	
//	Output:	-
//*****************************************************************************
void lcd_writestr(char * pc_str,unsigned char uc_zeile,unsigned char uc_offset)
{
unsigned char uc_x;

  if (uc_offset > LINECHARS) uc_offset = 0;
  if (uc_zeile == 1)
   {
     lcd_write(INSTRUCTION,FIRST_LINE + uc_offset);
   }

  if (uc_zeile == 2)
   {
     lcd_write(INSTRUCTION,SECOND_LINE + uc_offset);
   }	

  if (uc_zeile == 3)
   {
     lcd_write(INSTRUCTION,THIRD_LINE + uc_offset);
   }

  if (uc_zeile == 4)
   {
     lcd_write(INSTRUCTION,FOURTH_LINE + uc_offset);
   }

#ifdef OLDLCD	   
       timer0_delay(5);
#endif
   
  if (uc_zeile < 5)
   {  
     for(uc_x=0; uc_x < LINECHARS;uc_x++)
      {
        if (pc_str[uc_x] =='\0') break;
        lcd_write(DATA,pc_str[uc_x]);
      }			 
   } 		
}


//*****************************************************************************
//	LCD - Display loeschen
//  Input:  -	
//	Output:	-
//*****************************************************************************
void lcd_clear(void)
{
   lcd_write(INSTRUCTION,DISPLAY_CLEAR);
}


//*****************************************************************************
//	LCD - Display loeschen
//  Input:  Zeile
//	Output:	-
//*****************************************************************************
void lcd_lineclear(unsigned char uc_zeile)
{
unsigned char uc_x;

   if ((uc_zeile > 0) && (uc_zeile << 5))
    {
	   if (uc_zeile == 1) lcd_write(INSTRUCTION,FIRST_LINE);
	   if (uc_zeile == 2) lcd_write(INSTRUCTION,SECOND_LINE);
	   if (uc_zeile == 3) lcd_write(INSTRUCTION,THIRD_LINE);
	   if (uc_zeile == 4) lcd_write(INSTRUCTION,FOURTH_LINE);

#ifdef OLDLCD	   
       timer0_delay(5);
#endif
  
       for(uc_x=0; uc_x < LINECHARS;uc_x++)
       {
          lcd_write(DATA,0x20);
       }			 

	   if (uc_zeile == 1) lcd_write(INSTRUCTION,FIRST_LINE);
	   if (uc_zeile == 2) lcd_write(INSTRUCTION,SECOND_LINE);
	   if (uc_zeile == 3) lcd_write(INSTRUCTION,THIRD_LINE);
	   if (uc_zeile == 4) lcd_write(INSTRUCTION,FOURTH_LINE);	   	   
	}
}


//*****************************************************************************
//	LCD - setze Cursorposition
//  Input:  Spalte,Zeile
//	Output:	-
//*****************************************************************************
void lcd_goto(unsigned char uc_x,unsigned char uc_y)
{
unsigned char uc_n;

   if ((uc_y > 0) && (uc_y < 5))
    {
	   if (uc_x < LINECHARS)
	    {
 	       if (uc_y == 1) lcd_write(INSTRUCTION,FIRST_LINE);
 	       if (uc_y == 2) lcd_write(INSTRUCTION,SECOND_LINE);
	       if (uc_y == 3) lcd_write(INSTRUCTION,THIRD_LINE);
	       if (uc_y == 4) lcd_write(INSTRUCTION,FOURTH_LINE);
          		   
		   for(uc_n=0;uc_n < uc_x;uc_n++)
		    {
		       lcd_write(INSTRUCTION,CURSOR_RIGHT);
		    }  	   	   		   
		}
	}
}



