//****************************************************************************
//File name:	main.c
//Data:			23/10/2019
//Version:		1
//Company:		SLG
//Details: 		Display RS232
//				Language C: GCC compiler
//				Fosc = 16 MHz
// Fuse: 0xFD 0xDB 0xFF             
// DIV8 disable
//
//Author:	    N.Paasche (Dipl.-Ing.(FH) Ing.)	
//****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdlib.h>
#include <avr/wdt.h> 

#include "MCUinit.h"
#include "UART0.h"
#include "watchdog.h"
#include "timer0.h"
#include "LCD.H"


unsigned char uc_flag = 0, uc_charcnt = 0;
char pCHAR[8];
char c_nr;



//*****************************************************************************
//	Display Zeichen
//  Input:  c
//	Output:	-
//*****************************************************************************
void DisplayChar(char c_x)
{
   lcd_write(DATA,c_x);	
}


//*****************************************************************************
//	Display Steuerfunktion
//  Input:  c
//	Output:	-
//*****************************************************************************
void DisplayInit(char c_x)
{
	switch(c_x)
	{
	case 0x00: break; //Backlight off
	case 0x01: break; //Backlight on
	case 0x02: break; //Backlight on
	case 0x03: break; //Backlight on
	case 0x04: break; //Backlight on
	case 0x05: break; //Backlight on
	case 0x06: break; //Backlight on
	case 0x07: break; //Backlight on
	case 0x08: break; //Backlight on
	case 0x09: break; //Backlight on
	case 0x0A: break; //Backlight on
	case 0x0B: break; //Backlight on
	case 0x0C: break; //Backlight on
	case 0x0D: break; //Backlight on
	case 0x0E: break; //Backlight on
	case 0x0F: break; //Backlight on

	case 0x10: UBRR0  = 832; break; //Baudrate 1200
	case 0x11: UBRR0  = 416; break; //Baudrate 2400
	case 0x12: UBRR0  = 207; break; //Baudrate 4800
	case 0x13: UBRR0  = 103; break; //Baudrate 9600
	case 0x14: UBRR0  =  51; break; //Baudrate 19200
  							
	case 0x20: lcd_write(INSTRUCTION,FUNCTION_SET & 0x30); break; //LCD1
	case 0x21: lcd_write(INSTRUCTION,FUNCTION_SET); break; //LCD2

	default: break;	
	} //switch

	uc_flag = 0;
}


//*****************************************************************************
//	Display Steuerfunktion
//  Input:  c
//	Output:	-
//*****************************************************************************
void DisplayFunction(char c_x)
{
	switch(c_x)
	{
	case 0x00: lcd_write(INSTRUCTION,CURSOR_HOME); break;	//Home
	case 0x01: lcd_write(INSTRUCTION,DISPLAY_CLEAR); break;	//CLS	
	case 0x08: lcd_write(INSTRUCTION,DISPLAY_OFF); break;	//Blank
	case 0x0C: lcd_write(INSTRUCTION,DISPLAY_ON); 
			   lcd_write(INSTRUCTION,CURSOR_OFF); break;	//Cursor off
	case 0x0D: lcd_write(INSTRUCTION,CURSOR_BLINK2); break;	//Cursor on blink
	case 0x0E: lcd_write(INSTRUCTION,CURSOR_BLINK); break;	//Cursor on underline
	case 0x10: lcd_write(INSTRUCTION,CURSOR_LEFT); break;	//Move Cursor left
	case 0x14: lcd_write(INSTRUCTION,CURSOR_RIGHT); break;	//Move Cursor right
	case 0x18: lcd_write(INSTRUCTION,LINESHIFT_L); break;	//Scroll Display left
	case 0x1C: lcd_write(INSTRUCTION,LINESHIFT_R); break;	//Scroll Display right
	default:   if (c_x >= 0x80) lcd_write(INSTRUCTION,c_x); //Cursor auf Zeichenadresse setzen
			   break;
	} //switch	
	
	uc_flag = 0;
}


//*****************************************************************************
//	Display Sonderzeichen erzeugen
//  Input:  c
//	Output:	-
//*****************************************************************************
void DisplayNewChar(char c_x)
{
	switch(uc_charcnt)
	{	
	case 0: c_nr = c_x; break;
	case 1: pCHAR[0] = c_x; break;
	case 2: pCHAR[1] = c_x; break;
	case 3: pCHAR[2] = c_x; break;
	case 4: pCHAR[3] = c_x; break;
	case 5: pCHAR[4] = c_x; break;
	case 6: pCHAR[5] = c_x; break;
	case 7: pCHAR[6] = c_x; break;
	case 8: pCHAR[7] = c_x; 
			lcd_createchar(c_nr,pCHAR);
			uc_charcnt = 0;
			uc_flag = 0;
			break;
	default: uc_charcnt = 0;
			 uc_flag = 0;
			 break;		
	} //switch
	
    if (uc_flag) uc_charcnt++;	
}


//*****************************************************************************
//	Main - Startroutine
//  Input:	-
//	Output:	-
//*****************************************************************************
int main(void)
{
char c_char;
	
   MCUinit();					// MCU initialisieren
   timer0_init();
   UART0_INIT();

   lcd_init();

   watchdog_reset();   
   watchdog_set(0x04);
   watchdog_enable();
   
   lcd_clear();  
   uc_flag = 0;
   uc_charcnt = 0;


   //Endlosschleife
   while (1)
   {
      watchdog_reset();

      if (UART0_RXTest())
	   {
		  c_char = UART0_RX();
		  watchdog_reset();	 
		   
//		  UART0_TX(c_char);
//		  watchdog_reset();
     
		  if (uc_flag == 0) 
		   {
			  switch(c_char)
			  {
			  case 0xFD: uc_flag = 1; break;
			  case 0xFE: uc_flag = 2; break;
			  case 0xFF: uc_flag = 3; break; 
			  default:	 uc_flag = 0; break; 
			  } //switch

			  if (!uc_flag) 
			   {
				  watchdog_reset(); 
		          if ((c_char < 0x08) || 
				      ((c_char > 0x1F) && (c_char < 0x80)) ) DisplayChar(c_char);  //Zeichen ausgeben
			   }
		   }
		   else { //Steuercode senden
					watchdog_reset();
					
					switch(uc_flag)
					{
					case 1:  uc_charcnt = 0;
							 DisplayNewChar(c_char); break;	  	//0xFD
					case 2:  DisplayFunction(c_char); break;	//0xFE
					case 3:  DisplayInit(c_char); break;		//0xFF
					default: uc_flag = 0; break;	
					} //switch			        
				}
				
	   }
   } //while
}	



